@@ -5,8 +5,13 @@
 
 #![feature(const_fn, integer_atomics, staged_api, stdsimd)]
 #![feature(doc_cfg, allow_internal_unstable)]
-#![cfg_attr(feature = "cargo-clippy", allow(shadow_reuse))]
+#![cfg_attr(feature = "cargo-clippy", allow(clippy::shadow_reuse))]
+#![cfg_attr(
+    feature = "cargo-clippy",
+    deny(clippy::missing_inline_in_public_items,)
+)]
 #![cfg_attr(target_os = "linux", feature(linkage))]
+#![cfg_attr(all(target_os = "freebsd", target_arch = "aarch64"), feature(asm))]
 #![no_std]
 #![unstable(feature = "stdsimd", issue = "27731")]
 
